/**
 * @file python/storage.cxx 
 * @author <a href="mailto:andre.dos.anjos@cern.ch">Andre Anjos</a> 
 *
 * @brief Bindings to the EventStorage for reading and writing data
 */

#include <boost/python.hpp>
#include "eformat/python/util.h"
#include "eformat/SourceIdentifier.h"
#include "eformat/StreamTag.h"
#include "eformat/write/FullEventFragment.h"
#include "eformat/write/ROBFragment.h"

using namespace boost::python;

BOOST_PYTHON_MODULE(libpyeformat_write)
{
  class_<eformat::write::ROBFragment>("ROBFragment", init<>())
    .def(init<const eformat::write::ROBFragment&>())
    .def("copy_header", (void(eformat::write::ROBFragment::*)(const eformat::write::ROBFragment&))&eformat::write::ROBFragment::copy_header, with_custodian_and_ward<1, 2>())
    .def("copy_header", (void(*)(eformat::write::ROBFragment&, const eformat::read::ROBFragment&))&eformat::python::write_copy_header, with_custodian_and_ward<1, 2>())
    .def("copy_header", (void(*)(eformat::write::ROBFragment&, const eformat::helper::u32list&))&eformat::python::write_copy_header, with_custodian_and_ward<1, 2>())
    .def("copy_header", (void(*)(eformat::write::ROBFragment&, const eformat::helper::u32slice&))&eformat::python::write_copy_header, with_custodian_and_ward<1, 2>())
    .def("status", &eformat::python::write_get_status<eformat::write::ROBFragment>, with_custodian_and_ward_postcall<0, 1>())
    .def("status", (void(*)(eformat::write::ROBFragment&, eformat::helper::u32slice&))&eformat::python::write_set_status, with_custodian_and_ward<1, 2>())
    .def("status", (void(*)(eformat::write::ROBFragment&, eformat::helper::u32list&))&eformat::python::write_set_status, with_custodian_and_ward<1, 2>())
    .def("minor_version", (void(eformat::write::ROBFragment::*)(uint16_t))&eformat::write::ROBFragment::minor_version)
    .def("minor_version", (uint16_t(eformat::write::ROBFragment::*)(void) const)&eformat::write::ROBFragment::minor_version)
    .def("source_id", &eformat::python::write_get_source<eformat::write::ROBFragment>)
    .def("source_id", &eformat::python::write_set_source<eformat::write::ROBFragment>)
    .def("rod_minor_version", (void(eformat::write::ROBFragment::*)(uint16_t))&eformat::write::ROBFragment::rod_minor_version)
    .def("rod_minor_version", (uint16_t(eformat::write::ROBFragment::*)(void) const)&eformat::write::ROBFragment::rod_minor_version)
    .def("checksum_type", (void(eformat::write::ROBFragment::*)(uint32_t))&eformat::write::ROBFragment::checksum_type)
    .def("checksum_type", (uint32_t(eformat::write::ROBFragment::*)(void) const)&eformat::write::ROBFragment::checksum_type)
    .def("status_position", (void(eformat::write::ROBFragment::*)(uint32_t))&eformat::write::ROBFragment::status_position)
    .def("status_position", (uint32_t(eformat::write::ROBFragment::*)(void) const)&eformat::write::ROBFragment::status_position)
    .def("rod_status", &eformat::python::write_get_rod_status, with_custodian_and_ward_postcall<0, 1>())
    .def("rod_status", (void(*)(eformat::write::ROBFragment&, eformat::helper::u32slice&))&eformat::python::write_set_rod_status, with_custodian_and_ward<1, 2>())
    .def("rod_status", (void(*)(eformat::write::ROBFragment&, eformat::helper::u32list&))&eformat::python::write_set_rod_status, with_custodian_and_ward<1, 2>())
    .def("rob_source_id", &eformat::python::write_get_source<eformat::write::ROBFragment>)
    .def("rob_source_id", &eformat::python::write_set_source<eformat::write::ROBFragment>)
    .def("rod_source_id", &eformat::python::write_get_rod_source)
    .def("rod_source_id", &eformat::python::write_set_rod_source)
    .def("rod_run_no", (void(eformat::write::ROBFragment::*)(uint32_t))&eformat::write::ROBFragment::rod_run_no)
    .def("rod_run_no", (uint32_t(eformat::write::ROBFragment::*)(void) const)&eformat::write::ROBFragment::rod_run_no)
    .def("rod_lvl1_id", (void(eformat::write::ROBFragment::*)(uint32_t))&eformat::write::ROBFragment::rod_lvl1_id)
    .def("rod_lvl1_id", (uint32_t(eformat::write::ROBFragment::*)(void) const)&eformat::write::ROBFragment::rod_lvl1_id)
    .def("rod_bc_id", (void(eformat::write::ROBFragment::*)(uint32_t))&eformat::write::ROBFragment::rod_bc_id)
    .def("rod_bc_id", (uint32_t(eformat::write::ROBFragment::*)(void) const)&eformat::write::ROBFragment::rod_bc_id)
    .def("rod_lvl1_trigger_type", (void(eformat::write::ROBFragment::*)(uint32_t))&eformat::write::ROBFragment::rod_lvl1_type)
    .def("rod_lvl1_trigger_type", (uint32_t(eformat::write::ROBFragment::*)(void) const)&eformat::write::ROBFragment::rod_lvl1_type)
    .def("rod_detev_type", (void(eformat::write::ROBFragment::*)(uint32_t))&eformat::write::ROBFragment::rod_detev_type)
    .def("rod_detev_type", (uint32_t(eformat::write::ROBFragment::*)(void) const)&eformat::write::ROBFragment::rod_detev_type)
    .def("rod_data", &eformat::python::write_get_rod_data, with_custodian_and_ward_postcall<0, 1>())
    .def("rod_data", (void(*)(eformat::write::ROBFragment&, eformat::helper::u32slice&))&eformat::python::write_set_rod_data, with_custodian_and_ward<1, 2>())
    .def("rod_data", (void(*)(eformat::write::ROBFragment&, eformat::helper::u32list&))&eformat::python::write_set_rod_data, with_custodian_and_ward<1, 2>())
    .def("__len__", &eformat::write::ROBFragment::size_word)
    .def("meta_size_word", &eformat::write::ROBFragment::meta_size_word)
    .def("__raw__", &eformat::python::build<eformat::write::ROBFragment>)
    .def("parent", (const eformat::write::FullEventFragment*(eformat::write::ROBFragment::*)(void) const)&eformat::write::ROBFragment::parent, return_internal_reference<>())
    .def("next", (const eformat::write::ROBFragment*(eformat::write::ROBFragment::*)(void) const)&eformat::write::ROBFragment::next, return_internal_reference<>())
   ; 
  
  class_<eformat::write::FullEventFragment>("FullEventFragment", 
      init<uint32_t, uint32_t, uint32_t, uint64_t, uint32_t, uint32_t, uint16_t, uint32_t, uint16_t, uint8_t>())
    .def(init<>())
    .def(init<const eformat::write::FullEventFragment&>())
    .def("copy_header", (void(eformat::write::FullEventFragment::*)(const eformat::write::FullEventFragment&))&eformat::write::FullEventFragment::copy_header, with_custodian_and_ward<1, 2>())
    .def("copy_header", (void(*)(eformat::write::FullEventFragment&, const eformat::read::FullEventFragment&))&eformat::python::write_copy_header, with_custodian_and_ward<1, 2>())
    .def("copy_header", (void(*)(eformat::write::FullEventFragment&, const eformat::helper::u32list&))&eformat::python::write_copy_header, with_custodian_and_ward<1, 2>())
    .def("copy_header", (void(*)(eformat::write::FullEventFragment&, const eformat::helper::u32slice&))&eformat::python::write_copy_header, with_custodian_and_ward<1, 2>())
    .def("status", &eformat::python::write_get_status<eformat::write::FullEventFragment>, with_custodian_and_ward_postcall<0, 1>())
    .def("status", (void(*)(eformat::write::FullEventFragment&, eformat::helper::u32slice&))&eformat::python::write_set_status<eformat::write::FullEventFragment>, with_custodian_and_ward<1, 2>())
    .def("status", (void(*)(eformat::write::FullEventFragment&, eformat::helper::u32list&))&eformat::python::write_set_status<eformat::write::FullEventFragment>, with_custodian_and_ward<1, 2>())
    .def("minor_version", (void(eformat::write::FullEventFragment::*)(uint16_t))&eformat::write::FullEventFragment::minor_version)
    .def("minor_version", (uint16_t(eformat::write::FullEventFragment::*)(void) const)&eformat::write::FullEventFragment::minor_version)
    .def("source_id", &eformat::python::write_get_source<eformat::write::FullEventFragment>)
    .def("source_id", &eformat::python::write_set_source<eformat::write::FullEventFragment>)
    .def("checksum_type", (void(eformat::write::FullEventFragment::*)(uint32_t))&eformat::write::FullEventFragment::checksum_type)
    .def("checksum_type", (uint32_t(eformat::write::FullEventFragment::*)(void) const)&eformat::write::FullEventFragment::checksum_type)
    .def("bc_time_seconds", (void(eformat::write::FullEventFragment::*)(uint32_t))&eformat::write::FullEventFragment::bc_time_seconds)
    .def("bc_time_seconds", (uint32_t(eformat::write::FullEventFragment::*)(void) const)&eformat::write::FullEventFragment::bc_time_seconds)
    .def("bc_time_nanoseconds", (void(eformat::write::FullEventFragment::*)(uint32_t))&eformat::write::FullEventFragment::bc_time_nanoseconds)
    .def("bc_time_nanoseconds", (uint32_t(eformat::write::FullEventFragment::*)(void) const)&eformat::write::FullEventFragment::bc_time_nanoseconds)
    .def("global_id", (void(eformat::write::FullEventFragment::*)(uint64_t))&eformat::write::FullEventFragment::global_id)
    .def("global_id", (uint64_t(eformat::write::FullEventFragment::*)(void) const)&eformat::write::FullEventFragment::global_id)
    .def("run_type", (void(eformat::write::FullEventFragment::*)(uint32_t))&eformat::write::FullEventFragment::run_type)
    .def("run_type", (uint32_t(eformat::write::FullEventFragment::*)(void) const)&eformat::write::FullEventFragment::run_type)
    .def("run_no", (void(eformat::write::FullEventFragment::*)(uint32_t))&eformat::write::FullEventFragment::run_no)
    .def("run_no", (uint32_t(eformat::write::FullEventFragment::*)(void) const)&eformat::write::FullEventFragment::run_no)
    .def("lumi_block", (void(eformat::write::FullEventFragment::*)(uint16_t))&eformat::write::FullEventFragment::lumi_block)
    .def("lumi_block", (uint16_t(eformat::write::FullEventFragment::*)(void) const)&eformat::write::FullEventFragment::lumi_block)
    .def("lvl1_id", (void(eformat::write::FullEventFragment::*)(uint32_t))&eformat::write::FullEventFragment::lvl1_id)
    .def("lvl1_id", (uint32_t(eformat::write::FullEventFragment::*)(void) const)&eformat::write::FullEventFragment::lvl1_id)
    .def("bc_id", (void(eformat::write::FullEventFragment::*)(uint16_t))&eformat::write::FullEventFragment::bc_id)
    .def("bc_id", (uint16_t(eformat::write::FullEventFragment::*)(void) const)&eformat::write::FullEventFragment::bc_id)
    .def("run_type", (void(eformat::write::FullEventFragment::*)(uint32_t))&eformat::write::FullEventFragment::run_type)
    .def("run_type", (uint32_t(eformat::write::FullEventFragment::*)(void) const)&eformat::write::FullEventFragment::run_type)
    .def("lvl1_trigger_type", (void(eformat::write::FullEventFragment::*)(uint8_t))&eformat::write::FullEventFragment::lvl1_trigger_type)
    .def("compression_type", (void(eformat::write::FullEventFragment::*)(uint32_t))&eformat::write::FullEventFragment::compression_type)
    .def("compression_type", (uint32_t(eformat::write::FullEventFragment::*)(void) const)&eformat::write::FullEventFragment::compression_type)
    .def("compression_level", (void(eformat::write::FullEventFragment::*)(unsigned int))&eformat::write::FullEventFragment::compression_level)
    .def("compression_level", (uint32_t(eformat::write::FullEventFragment::*)(void) const)&eformat::write::FullEventFragment::compression_level)
    .def("lvl1_trigger_type", (uint8_t(eformat::write::FullEventFragment::*)(void) const)&eformat::write::FullEventFragment::lvl1_trigger_type)
    .def("lvl1_trigger_info", &eformat::python::write_get_lvl1_trigger_info, with_custodian_and_ward_postcall<0, 1>())
    .def("lvl1_trigger_info", (void(*)(eformat::write::FullEventFragment&, eformat::helper::u32slice&))&eformat::python::write_set_lvl1_trigger_info, with_custodian_and_ward<1, 2>())
    .def("lvl1_trigger_info", (void(*)(eformat::write::FullEventFragment&, eformat::helper::u32list&))&eformat::python::write_set_lvl1_trigger_info, with_custodian_and_ward<1, 2>())
    .def("lvl2_trigger_info", &eformat::python::write_get_lvl2_trigger_info, with_custodian_and_ward_postcall<0, 1>())
    .def("lvl2_trigger_info", (void(*)(eformat::write::FullEventFragment&, eformat::helper::u32slice&))&eformat::python::write_set_lvl2_trigger_info, with_custodian_and_ward<1, 2>())
    .def("lvl2_trigger_info", (void(*)(eformat::write::FullEventFragment&, eformat::helper::u32list&))&eformat::python::write_set_lvl2_trigger_info, with_custodian_and_ward<1, 2>())
    .def("event_filter_info", &eformat::python::write_get_event_filter_info, with_custodian_and_ward_postcall<0, 1>())
    .def("event_filter_info", (void(*)(eformat::write::FullEventFragment&, eformat::helper::u32slice&))&eformat::python::write_set_event_filter_info, with_custodian_and_ward<1, 2>())
    .def("event_filter_info", (void(*)(eformat::write::FullEventFragment&, eformat::helper::u32list&))&eformat::python::write_set_event_filter_info, with_custodian_and_ward<1, 2>())
    .def("stream_tag", &eformat::python::write_get_stream_tag)
    .def("raw_stream_tag", &eformat::python::write_get_raw_stream_tag, with_custodian_and_ward_postcall<0, 1>())
    .def("stream_tag", (void(*)(eformat::write::FullEventFragment&, eformat::helper::u32slice&))&eformat::python::write_set_stream_tag, with_custodian_and_ward<1, 2>())
    .def("stream_tag", (void(*)(eformat::write::FullEventFragment&, eformat::helper::u32list&))&eformat::python::write_set_stream_tag, with_custodian_and_ward<1, 2>())
    .def("__len__", &eformat::write::FullEventFragment::size_word)
    .def("meta_size_word", &eformat::write::FullEventFragment::meta_size_word)
    .def("append", &eformat::write::FullEventFragment::append, with_custodian_and_ward<1, 2>())
    .def("__raw__", &eformat::python::build<eformat::write::FullEventFragment>)
    .def("first_child", &eformat::write::FullEventFragment::first_child, return_internal_reference<>())
    .def("append_unchecked", (void(*)(eformat::write::FullEventFragment&, eformat::helper::u32list&))&eformat::python::write_append_unchecked, with_custodian_and_ward<1,2>()) 
    .def("append_unchecked", (void(*)(eformat::write::FullEventFragment&, eformat::helper::u32slice&))&eformat::python::write_append_unchecked, with_custodian_and_ward<1,2>()) 
    .def("append_unchecked", (void(*)(eformat::write::FullEventFragment&, eformat::read::ROBFragment&))&eformat::python::write_append_unchecked, with_custodian_and_ward<1,2>())
    .def("nunchecked_fragments", &eformat::write::FullEventFragment::nunchecked_fragments)
    .def("unchecked_fragment", &eformat::python::unchecked_fragment, with_custodian_and_ward_postcall<0, 1>())
   ; 
}
